@@ -10,12 +10,15 @@ class Annotation
 {
 public:
     Annotation(int, float, QString, QString, QString, QColor);
-    const QString& getAnnotation();
-    const QString& getParagraph();
-    const QString& getText();
-    int getSection();
-    float getPercentage();
-    const QColor& getColor();
+    const QString& getAnnotation() const;
+    const QString& getParagraph() const;
+    const QString& getText() const;
+    int getSection() const;
+    float getPercentage() const;
+    const QColor& getColor() const;
+
+    bool operator==(const Annotation&) const;
+    bool operator!=(const Annotation&) const;
 
 private:
     int section;
