@@ -5,6 +5,11 @@
 const QString dummyURL = QString("ebook://get.com");
 const QString OSSO_NAME = QString("mebook_osso");
 enum BookViewMode{ScrollingMode, PageMode};
+enum Sorting{
+    byAuthor,
+    byTitle,
+    None
+};
     
 
 
