@@ -16,13 +16,19 @@
 #include <QMenu>
 #include <QMessageBox>
 #include <QGraphicsProxyWidget>
+#include <QGraphicsLinearLayout>
+#include <QMaemo5InformationBox>
+#include <QDBusConnection>
+#include <QDBusMessage>
 
 #include "addbookmarkdialog.h"
 #include "addannotationdialog.h"
+#include "listviewdialog.h"
 
 #include "libosso.h"
 
 //Q_DECLARE_METATYPE(RenderTile)
+const int topMargin = 90;
 
 BookView::BookView(QWidget *parent) : QGraphicsView(parent), 
         autoScrollStep(1), 
@@ -42,7 +48,9 @@ BookView::BookView(QWidget *parent) : QGraphicsView(parent),
     pageWidgetCurrent = NULL;
     pageWidgetNext = NULL;
     scrollablePage = NULL;
-    
+
+
+
 
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -66,8 +74,6 @@ BookView::BookView(QWidget *parent) : QGraphicsView(parent),
 
     setContextMenuPolicy(Qt::CustomContextMenu);
     connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(openContextMenu(const QPoint&)));
-
-
     
 }
 
@@ -129,9 +135,9 @@ void BookView::goToSection(int section, float multiplier)
             currentPosition.multiplier = multiplier;
             currentPosition.page = -1;
             pageWidgetCurrent->setPixmap(loadingPixmap());
+            loadingBar->setValue(0);
+            loadingBarProxy->setVisible(true);
             deleteTiles();
-            if(multiplier == 0)
-                progressSlider->setValue(book->getSectionStart(currentPosition.section.number));
             emit requestRender(content, currentPosition.section.number);
         }
     }else{
@@ -140,6 +146,9 @@ void BookView::goToSection(int section, float multiplier)
         currentPosition.multiplier = multiplier;
         emit requestRender(content, currentPosition.section.number);
     }
+
+    int value = ((float)book->getSectionLength(currentPosition.section.number) * ((float)currentPosition.multiplier)) + book->getSectionStart(currentPosition.section.number);
+    progressSlider->setValue(value);
 }
 
 void BookView::goToTOCIndex(int index)
@@ -205,8 +214,8 @@ void BookView::setNightMode(bool toggle)
     }
     else{
         renderSettings.setTextColor(QColor(Qt::black));
-        renderSettings.setBackgroundColor(QColor(Qt::lightGray));
-        scene->setBackgroundBrush(QBrush(Qt::lightGray));
+        renderSettings.setBackgroundColor(QColor("#EDEADB"));
+        scene->setBackgroundBrush(QBrush(QColor("#EDEADB")));
     }
     
 }
@@ -222,20 +231,19 @@ void BookView::setViewMode(BookViewMode mode, Qt::WidgetAttribute orientation)
     int screenHeight = 0;
     if(orientation == Qt::WA_Maemo5PortraitOrientation){
         pageWidth = 480;
-        pageHeight = 730;
+        pageHeight = 730 - 70;
 
         screenWidth = 480;
         screenHeight = 800;
     }
     else{
         pageWidth = 800;
-        pageHeight = 410;
+        pageHeight = 400 - 70;
 
         screenWidth = 800;
         screenHeight = 480;
     }
 
-    const int topMargin = 20;
 
     if(viewMode == PageMode){
 
@@ -252,11 +260,8 @@ void BookView::setViewMode(BookViewMode mode, Qt::WidgetAttribute orientation)
 
         deleteTiles();
 
-        pageWidgetCurrent->setPos(0, 0);
-        pageWidgetNext->setPos(pageWidth, 0);
-
-        pageWidgetCurrent->setOffset(0, topMargin);
-        pageWidgetNext->setOffset(0, topMargin);
+        pageWidgetCurrent->setPos(0, topMargin);
+        pageWidgetNext->setPos(pageWidth, topMargin);
 
         pageWidgetCurrent->setPixmap(loadingPixmap());
 
@@ -272,7 +277,7 @@ void BookView::setViewMode(BookViewMode mode, Qt::WidgetAttribute orientation)
         scene->addItem(scrollablePage);
         connect(renderer, SIGNAL(webPageLoaded(QWebPage*)), this, SLOT(webPageLoaded(QWebPage*)));
         connect(scrollablePage, SIGNAL(positionChanged(int)), this, SLOT(sectionScrolled(int)));
-        scrollablePage->setGeometry(0, 0, pageWidth, pageHeight);
+        scrollablePage->setGeometry(0, topMargin, pageWidth, pageHeight);
         scene->setSceneRect(0, 0, screenWidth, screenHeight);
         renderer->enablePagination(false);
         renderer->enablePixmapRendering(false);
@@ -282,15 +287,79 @@ void BookView::setViewMode(BookViewMode mode, Qt::WidgetAttribute orientation)
     const int sliderHeight = 32;
     const int leftMargin = 5;
 
+
     progressSlider = new BookProgressSlider();
     progressSlider->setMinimumWidth(pageWidth - 10);
     progressSlider->setMaximum(100);
-    progressSlider->setColors(renderSettings.getTextColor(), renderSettings.getBackgroundColor());
+
+    progressSlider->setColors(QColor("#3d342e"), renderSettings.getBackgroundColor());
 
     connect(progressSlider, SIGNAL(sliderReleased()), this, SLOT(sliderMoved()));
     sliderProxy = scene->addWidget(progressSlider);
     sliderProxy->setPos(leftMargin, screenHeight - (sliderHeight + leftMargin));
 
+    loadingBar = new QProgressBar();
+    loadingBar->setStyleSheet("QProgressBar{ border: 2px solid #3d342e; border-radius: 10px; background: " + renderSettings.getBackgroundColor().name() + "} QProgressBar::chunk{ background: #3d342e; }");
+    loadingBar->setMaximumSize(QSize(240, 20));
+    loadingBar->setMinimumSize(QSize(240, 20));
+    loadingBar->setTextVisible(false);
+    loadingBar->setMinimum(0);
+
+
+    loadingBarProxy = scene->addWidget(loadingBar);
+    loadingBarProxy->setPos((screenWidth / 2) - 120, (screenHeight / 2) - 10);
+    loadingBarProxy->setVisible(false);
+
+    toolbarWidget = new GraphicsWidget();
+    QPalette palette;
+    palette.setColor(QPalette::Window, renderSettings.getBackgroundColor());
+    toolbarWidget->setPalette(palette);
+    QGraphicsLinearLayout *toolbarLayout = new QGraphicsLinearLayout();
+    toolbarWidget->setMinimumWidth(screenWidth);
+    toolbarWidget->setMinimumHeight(70);
+    toolbarLayout->setPreferredSize(screenWidth, 70);
+    toolbarLayout->setMinimumWidth(screenWidth);
+
+    wmDashboardIcon = new ToolbarIcon(":/wmdashboardicon_normal.png", ":/wmdashboardicon_pressed.png");
+    connect(wmDashboardIcon, SIGNAL(released()), this, SLOT(wmDashboard()));
+    wmBackIcon = new ToolbarIcon(":/wmbackicon_normal.png", ":/wmbackicon_pressed.png");
+    connect(wmBackIcon, SIGNAL(released()), this, SLOT(wmClose()));
+
+    TOCIcon = new ToolbarIcon(":/tocicon_normal.png", ":/tocicon_pressed.png");
+    connect(TOCIcon, SIGNAL(released()), this, SLOT(tocNavigation()));
+
+    bookmarkIcon = new ToolbarIcon(":/bookmarkicon_normal.png", ":/bookmarkicon_pressed.png");
+    connect(bookmarkIcon, SIGNAL(released()), this, SLOT(viewBookmarks()));
+
+    annotationIcon = new ToolbarIcon(":/annotationicon_normal.png", ":/annotationicon_pressed.png");
+    connect(annotationIcon, SIGNAL(released()), this, SLOT(viewAnnotations()));
+
+    historyBackIcon = new ToolbarIcon(":/historybackicon_normal.png", ":/historybackicon_pressed.png");
+    connect(historyBackIcon, SIGNAL(released()), this, SLOT(backInHistory()));
+
+    nextSectionIcon = new ToolbarIcon(":/nextsectionicon_normal.png", ":/nextsectionicon_pressed.png");
+    connect(nextSectionIcon, SIGNAL(released()), this, SLOT(nextSection()));
+    previousSectionIcon = new ToolbarIcon(":/previoussectionicon_normal.png", ":/previoussectionicon_pressed.png");
+    connect(previousSectionIcon, SIGNAL(released()), this, SLOT(previousSection()));
+
+    toolbarLayout->addItem(wmDashboardIcon);
+    toolbarLayout->addItem(TOCIcon);
+    if(viewMode == PageMode){
+        toolbarLayout->addItem(historyBackIcon);
+    } else {
+        toolbarLayout->addItem(previousSectionIcon);
+        toolbarLayout->addItem(nextSectionIcon);
+    }
+    toolbarLayout->addItem(bookmarkIcon);
+    toolbarLayout->addItem(annotationIcon);
+    toolbarLayout->addItem(wmBackIcon);
+    
+
+    toolbarWidget->setLayout(toolbarLayout);
+
+    scene->addItem(toolbarWidget);
+    toolbarWidget->setPos(0, 0);
+
     pageSize = QSizeF(pageWidth, pageHeight); 
     renderer->setPageSize(pageSize);
     renderSettings.setMaxImageSize(pageWidth-(2*leftMargin), pageHeight-(2*topMargin));
@@ -299,7 +368,6 @@ void BookView::setViewMode(BookViewMode mode, Qt::WidgetAttribute orientation)
 
 void BookView::addBookmark(QString name, QColor color)
 {
-    //TODO:Fix pagemode multiplier..
     float multiplier = currentPosition.multiplier;
     if(viewMode == PageMode) 
         multiplier = ((float)currentPosition.page / ((float)currentPosition.section.numPages));
@@ -388,9 +456,9 @@ void BookView::mouseReleaseEvent(QMouseEvent *event)
         QString annotationHit;
         bool hit = false;
         if(viewMode == PageMode){
-            hit = renderer->hitAnnotation(event->pos() - QPoint(0, 20), annotationHit, tiles[currentPosition.section.number][currentPosition.page]);
+            hit = renderer->hitAnnotation(event->pos() - QPoint(0, topMargin), annotationHit, tiles[currentPosition.section.number][currentPosition.page]);
         } else{
-            QPoint t = QPoint(event->pos().x(), scrollablePage->getPosition() + event->pos().y());
+            QPoint t = QPoint(event->pos().x(), scrollablePage->getPosition() + event->pos().y() + topMargin);
             hit = renderer->hitAnnotation(t, annotationHit);
         }
 
@@ -410,16 +478,18 @@ void BookView::mouseReleaseEvent(QMouseEvent *event)
     QGraphicsView::mouseReleaseEvent(event);
 }
 
-void BookView::resizeEvent(QResizeEvent *event)
+
+bool BookView::eventFilter(QObject *object, QEvent *event)
 {
-    //Make sure pageWidget stays fully visible when returning to application from app switcher
-/*    QPointF viewPos = mapToScene(pos());
-    if(viewPos != QPointF(0, 0)){
-        QPointF move = QPointF(0, 0) - viewPos;
-        translate(viewPos.x(), viewPos.y());
+    if(event->type() == QEvent::Close){
+        delete object;
+//        parentWidget()->showNormal();
+        QApplication::setActiveWindow(parentWidget());
+        parentWidget()->setFocus();
+        return true;
     }
-    ensureVisible(QRectF(0, 0, pageSize.width(), pageSize.height()));*/
-    QGraphicsView::resizeEvent(event);
+
+    return QObject::eventFilter(object, event);
 }
 
 
@@ -488,8 +558,11 @@ QPixmap BookView::getCurrentPagePixmap()
 {
     if(tiles.contains(currentPosition.section.number)){
         QHash<int, RenderTile*> pages = tiles[currentPosition.section.number];
-        if(pages.contains(currentPosition.page))
+        if(pages.contains(currentPosition.page)){
+            if(!pages[currentPosition.page]->isRendered())
+                renderer->renderTile(pages[currentPosition.page]);
             return QPixmap::fromImage(pages[currentPosition.page]->getPixmap());
+        }
     }    
     
     return loadingPixmap();
@@ -500,9 +573,13 @@ QPixmap BookView::getNextPagePixmap()
     if(tiles.keys().contains(currentPosition.section.number)){
         QHash<int, RenderTile*> pages = tiles[currentPosition.section.number];
         if(pages.contains(currentPosition.page+1)){
+            if(!pages[currentPosition.page+1]->isRendered())
+                renderer->renderTile(pages[currentPosition.page+1]);
             return QPixmap::fromImage(pages[currentPosition.page+1]->getPixmap());
         }else if(tiles.keys().contains(currentPosition.section.number+1)){
             pages = tiles[currentPosition.section.number+1];
+            if(!pages[0]->isRendered())
+                renderer->renderTile(pages[0]);
             return QPixmap::fromImage(pages[0]->getPixmap());
         }
     }
@@ -514,6 +591,8 @@ QPixmap BookView::getPreviousPagePixmap()
     if(tiles.keys().contains(currentPosition.section.number)){
         QHash<int, RenderTile*> pages = tiles[currentPosition.section.number];
         if(pages.contains(currentPosition.page-1)){
+            if(!pages[currentPosition.page-1]->isRendered())
+                renderer->renderTile(pages[currentPosition.page-1]);
             return QPixmap::fromImage(pages[currentPosition.page-1]->getPixmap());
         }
     }
@@ -574,13 +653,21 @@ void BookView::addTile(RenderTile *tile)
     }
 
     if(tile->getID() == currentPosition.page && tile->getSection() == currentPosition.section.number){
-        pageWidgetCurrent->setPixmap(QPixmap::fromImage(tile->getPixmap()));
+//        pageWidgetCurrent->setPixmap(getCurrentPagePixmap());
     }
     
     //Set first page as cover if no cover image found.
-    if(tile->getSection() == 0 && tile->getID() == 0 && book->getCoverImage().isNull()){
+   /* if(tile->getSection() == 0 && tile->getID() == 0 && book->getCoverImage().isNull()){
         book->addCoverImage(QPixmap::fromImage(tile->getPixmap()));
+    }*/
+    loadingBar->setValue(tile->getID() + 1);
+    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+
+    if(tile->getID() >= loadingBar->maximum()- 1){
+        loadingBarProxy->setVisible(false);
     }
+
+
 }
 
 void BookView::contentHeight(int height)
@@ -590,6 +677,8 @@ void BookView::contentHeight(int height)
             currentPosition.page = ((float)height / pageSize.height()) * ((float)currentPosition.multiplier);
             currentPosition.section.contentHeight = height;
             currentPosition.section.numPages = (float)height / (float)pageSize.height();
+
+            loadingBar->setMaximum(currentPosition.section.numPages);
         }
         
     }else{
@@ -691,3 +780,119 @@ void BookView::sliderMoved()
     }
 
 }
+
+void BookView::tocNavigation()
+{
+    QList<QString> toc = book->getTOC();
+    if(toc.isEmpty()){
+        QMaemo5InformationBox::information(this, tr("This book doesn't have table of contents information."), QMaemo5InformationBox::DefaultTimeout); 
+        return;
+    }
+    //TODO: Show notofication banner if no TOC
+    QList<ListItem > list;
+    for(QList<QString>::iterator iter = toc.begin(); iter != toc.end(); ++iter){
+        ListItem item;
+        item.text = *iter;
+        list.push_back(item);
+    }
+
+    ListViewDialog dialog(this);
+    dialog.setWindowTitle(tr("Table of content"));
+    ListModel *model = new ListModel(list);
+    dialog.setModel(model);
+    dialog.setAttribute(Qt::WA_Maemo5StackedWindow);
+    dialog.setWindowFlags(dialog.windowFlags() | Qt::Window);
+    connect(&dialog, SIGNAL(selectedIndex(int)), this, SLOT(goToTOCIndex(int)));
+//    dialog.installEventFilter(this);
+    dialog.exec();
+}
+
+void BookView::viewBookmarks()
+{
+    QList<Bookmark> bm = book->getBookmarks();
+    if(bm.isEmpty()){
+        QMaemo5InformationBox::information(this, tr("No bookmarks found. Press and hold to add a bookmark."), QMaemo5InformationBox::DefaultTimeout); 
+        return;
+    }
+    QList<ListItem > list;
+    for(QList<Bookmark>::iterator iter = bm.begin(); iter != bm.end(); ++iter){
+        Bookmark bookmark = *iter;
+        QPixmap ip(32, 32); 
+        ip.fill(Qt::transparent);
+        QPainter painter(&ip);
+        painter.setRenderHint(QPainter::Antialiasing, true);
+        painter.setPen(Qt::NoPen);
+        painter.setBrush(bookmark.getColor());
+        painter.drawRoundedRect(ip.rect(), 3, 3);
+        ListItem item;
+        item.text = bookmark.getName();
+        QString sectionName = book->getSectionName(bookmark.getSection());
+        if(sectionName.isEmpty())
+            sectionName = tr("Section") + " " + QString::number(bookmark.getSection());
+        item.subText =  sectionName + ", " + bookmark.percentageAsString() + "%";
+        item.icon = ip;
+
+        list.push_back(item);
+    }
+    
+    ListViewDialog dialog(this, true);
+    dialog.setWindowTitle(tr("Bookmarks"));
+    ListModel *model = new ListModel(list);
+    dialog.setModel(model);
+    dialog.setAttribute(Qt::WA_Maemo5StackedWindow);
+    dialog.setWindowFlags(dialog.windowFlags() | Qt::Window);
+    connect(&dialog, SIGNAL(selectedIndex(int)), this, SLOT(goToBookmark(int)));
+    connect(&dialog, SIGNAL(deleteIndex(int)), book, SLOT(removeBookmark(int)));
+    dialog.exec();
+}
+
+
+void BookView::viewAnnotations()
+{
+    QList<Annotation> a = book->getAnnotations();
+    if(a.isEmpty()){
+        
+        QMaemo5InformationBox::information(this, tr("No annotations found. Press and hold paragraph to add an annotation."), QMaemo5InformationBox::DefaultTimeout); 
+        return;
+    }
+    QList<ListItem> list;
+    for(QList<Annotation>::iterator iter = a.begin(); iter != a.end(); ++iter){
+        Annotation annotation = *iter;
+        QPixmap ip(32, 32); 
+        ip.fill(Qt::transparent);
+        QPainter painter(&ip);
+        painter.setRenderHint(QPainter::Antialiasing, true);
+        painter.setPen(Qt::NoPen);
+        painter.setBrush(annotation.getColor());
+        painter.drawRoundedRect(ip.rect(), 3, 3);
+
+        ListItem item;
+        item.icon = ip;
+        item.text = annotation.getAnnotation();
+        item.subText = book->getSectionName(annotation.getSection());
+
+        list.push_back(item);
+    }
+    ListViewDialog dialog(this, true);
+    dialog.setWindowTitle(tr("Annotations"));
+    ListModel *model = new ListModel(list);
+    dialog.setModel(model);
+    connect(&dialog, SIGNAL(selectedIndex(int)), this, SLOT(goToAnnotation(int)));
+    connect(&dialog, SIGNAL(deleteIndex(int)), book, SLOT(removeAnnotation(int)));
+    dialog.setAttribute(Qt::WA_Maemo5StackedWindow);
+    dialog.setWindowFlags(dialog.windowFlags() | Qt::Window);
+    dialog.exec();
+    
+}
+
+void BookView::wmDashboard()
+{ 
+    QDBusConnection c = QDBusConnection::sessionBus();
+    QDBusMessage m = QDBusMessage::createSignal("/", "com.nokia.hildon_desktop", "exit_app_view");
+    c.send(m);
+}
+
+void BookView::wmClose()
+{
+    emit closeWindow();
+}
