@@ -8,7 +8,7 @@
 #include "bookview.h"
 
 //MainWindow is the mainwindow that has the reading UI. Contains toolbar buttons for TOC, Bookmarks and Annotations. 
-class MainWindow : public QMainWindow{
+class MainWindow : public QWidget{
     Q_OBJECT
 public:
     MainWindow(osso_context_t *context = 0, QWidget *parent = 0);
@@ -24,23 +24,18 @@ public:
     void setViewMode(BookViewMode, Qt::WidgetAttribute);
 
 protected:
-    bool eventFilter(QObject*, QEvent*);
     void keyPressEvent(QKeyEvent*);
     void closeEvent(QCloseEvent*);
 
 private:
     BookView *bookView;
     Book *currentBook;
-    QToolBar *toolbar;
     void grabZoomKeys(bool);
     QTimer brightnessTimer;
-    //osso_context is used for in forcing the backlight on.
+    //osso_context is used in forcing the backlight on.
     osso_context_t *osso_context;
 
 public slots:
-    void tocNavigation();
-    void viewBookmarks();
-    void viewAnnotations();
     void forceBacklightOn();
 
 signals:
