@@ -12,7 +12,7 @@
 #include "settingsdialog.h"
 #include "searchcatalogsview.h"
 
-//StartWindow is the mainwindow visible when the application first starts.
+//StartWindow is the mainwindow visible when the application starts.
 //It contains menu for settings, button for opening files, buttons for accessing catalogs and
 //a list of books in library.
 class StartWindow : public QMainWindow
@@ -26,10 +26,11 @@ private:
     QScrollArea *scrollArea;
     QWidget *bookList;
     QVBoxLayout *listLayout;
+    QVBoxLayout *bookListLayout;
     QAbstractKineticScroller *scroller;
 
     MainWindow *readingUI;
-    SearchCatalogsView *searcCatalogsView;
+    SearchCatalogsView *searchCatalogsView;
     QListWidget *libraryList;
     QSettings *settings;
 
@@ -41,6 +42,14 @@ private:
 
     QList<BookListItem*> bookListItems;
 
+    QStringList authors;
+    QStringList titles;
+    Sorting sorting;
+    void sortBooks(Sorting);
+    QAction *sortByAuthorAction;
+    QAction *sortByTitleAction;
+    QAction *sortByDateAddedAction;
+
 
 public slots:
     void openFile();
@@ -60,6 +69,13 @@ public slots:
 
     void reportBugs();
 
+    void openContextMenu(const QPoint&);
+
+    void sortByAuthor();
+    void sortByTitle();
+    void sortByDateAdded();
+    void search(const QString&);
+
 };
 
 #endif
